// Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2021, 2023 PISM Authors
//
// This file is part of PISM.
//
// PISM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// PISM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License
// along with PISM; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#ifndef _PSANOMALY_H_
#define _PSANOMALY_H_

#include "pism/coupler/SurfaceModel.hh"

namespace pism {
namespace surface {

//! @brief Reads and uses climatic_mass_balance and ice_surface_temp *anomalies* from a
//! file.
class Anomaly : public SurfaceModel
{
public:
  Anomaly(std::shared_ptr<const Grid> g, std::shared_ptr<SurfaceModel> in);
  virtual ~Anomaly() = default;
protected:
  virtual void init_impl(const Geometry &geometry);
  virtual void update_impl(const Geometry &geometry, double t, double dt);

  const array::Scalar& mass_flux_impl() const;
  const array::Scalar& temperature_impl() const;
  const array::Scalar& accumulation_impl() const;
  const array::Scalar& melt_impl() const;
  const array::Scalar& runoff_impl() const;
protected:
  std::shared_ptr<array::Scalar> m_mass_flux;
  std::shared_ptr<array::Scalar> m_temperature;

  std::shared_ptr<array::Forcing> m_climatic_mass_balance_anomaly;
  std::shared_ptr<array::Forcing> m_ice_surface_temp_anomaly;
};

} // end of namespace surface
} // end of namespace pism

#endif /* _PSANOMALY_H_ */
